; COLTEMP.PRO
; Calculation of color temperature and "true" intensities and sizes
; of umbral dots (UDs). Based on the method published by Aballe Villero
; in his thesis (La Laguna, 1992).
;
; SYNTAX:  coltemp,tug,tuc,mu,tru
;
; INPUTS: TUG, TUC = arrays/tables resulting from UDSEGM.PRO
;	  MU = cos(theta), heliocentric position
; OUTPUT: TRU = 2D float array/table (13,n_UDs) with:
;	 0: x-position (average of 'gc' and 'c6' x-positions)
;	 1: y-position (average of 'gc' and 'c6' y-positions)
;	 2: observed UD intensity in 'gc'
;	 3: observed UD intensity in 'c6'
;	 4: observed local background intensity in 'gc'
;	 5: observed local background intensity in 'c6'
;	 6: color index
;	 7: color temperature (K)
;	 8: true UD intensity in 'gc'
;	 9: true UD intensity in 'c6'
;	10: observed UD effective diameter in 'gc' (")
;	11: observed UD effective diameter in 'c6' (")
;	12: true UD diameter (")
;	NOTE: All intensities are in units of the local Iph=1
;
; Routines called: CLV, NINT, RTBIS, TEQ (in this file)
; Used for analysis of the SST 2003 observations.
; Michal, 13.4.2004.
;
; ================ FUNCTION TEQ =====================
;
FUNCTION teq,x
;
; Used for finding the root of the non-linear equation
; of color temperature.
; The root X is the true 'c6' intensity excess.
;
 COMMON SHR,c1,wav1,wav2,iph1,iph2,idb1,idb2,ci

 a1=alog(c1/(wav1^5*iph1*(idb1+ci*x))+1.)
 a2=alog(c1/(wav2^5*iph2*(idb2+x))+1.)
 RETURN,wav1*a1-wav2*a2

END
;
; ================ PROCEDURE COLTEMP ===============
;
PRO coltemp,tug,tuc,mu,tru
;
; See comments at the beginning of the file.
; tug, tuc must have equal dimensions
;
on_error,1

 COMMON SHR,c1,wav1,wav2,iph1,iph2,idb1,idb2,ci

nuds=(size(tug))(2)	; number of UDs
tru=fltarr(13,nuds)
print,' Number of objects:',nuds

; Constants ---------------------------------------------

norm =10000.	        ; input intensity normalization
scale=0.041	        ; image scale (arcsec/pix)
wav1 =4507.5	        ; wavelength 'gc'
wav2 =6020.0	        ; wavelength 'c6'
iph1o=0.4138	        ; disk-center absolute intensity for 'gc'
iph2o=0.3095    	; disk-center absolute intensity for 'c6'

x1=0.001        ; minimum for true 'c6' intensity excess
x2=2.           ; maximum for true 'c6' intensity excess
c1=1.1903e20	; Planck's law
c2=1.4384e8
ss=string(byte([27,91,68,27,91,68,27,91,68])) ;for displaying

; -------------------------------------------------------
; Absolute intensities at the position 'mu'

iph1=iph1o*CLV(-1,wav1,mu)
iph2=iph2o*CLV(-1,wav2,mu)

; LOOP over UDs (lines in tug and tuc)

for i=0,nuds-1 do begin
  writeu,-1,ss+string(i,format='(I3)')	;displaying 

; Average coordinates

  tru(0,i)=nint((tug(0,i)+tuc(0,i))/2.)
  tru(1,i)=nint((tug(1,i)+tuc(1,i))/2.)

; Intensity normalization and color index

  tru(2,i)=tug(2,i)/norm	;UDs
  tru(3,i)=tuc(2,i)/norm
  idb1=tug(4,i)/norm		;background
  idb2=tuc(4,i)/norm
  tru(4,i)=idb1
  tru(5,i)=idb2
  di1=tru(2,i)-idb1		;UD intensity excess
  di2=tru(3,i)-idb2
  ci=(di1*tug(3,i))/(di2*tuc(3,i))	;color index
  tru(6,i)=ci
  tru(10,i)=sqrt(4.*tug(3,i)/!pi)*scale	;observed diameter
  tru(11,i)=sqrt(4.*tuc(3,i)/!pi)*scale 

; Solving color-temperature equation

  dj2=RTBIS(x1,x2,'TEQ',flg,1.e-5)
  if flg eq 0 then goto,skip

; Calculation of true intensities, diameters, and color temperature

  tru(8,i)=idb1+ci*dj2		;true intensity 'gc'
  tru(9,i)=idb2+dj2		;true intensity 'c6'
  are=tuc(3,i)*di2/dj2
  tru(12,i)=sqrt(4.*are/!pi)*scale  ;true diameter
  tru(7,i)=c2/(wav2*alog(c1/(wav2^5*iph2*tru(9,i))+1))  ;temperature
  skip:

endfor
print,''

; Removing UDs where the color temperature was not determined

w=where(tru(7,*),cnt)
tru=tru(*,w)
print,' Final number of objects:',cnt

END

